@@ -51,6 +51,8 @@ cpl_array *cpl_array_wrap_float_complex(_Complex float *, cpl_size);
 cpl_array *cpl_array_wrap_double_complex(_Complex double *, cpl_size);
 #endif
 cpl_array *cpl_array_wrap_string(char **, cpl_size);
+cpl_array *cpl_array_new_complex_from_arrays(const cpl_array *,
+                                             const cpl_array *) CPL_ATTR_ALLOC;
 
 cpl_error_code cpl_array_copy_data(cpl_array *, const double *);
 cpl_error_code cpl_array_copy_data_int(cpl_array *, const int *);
@@ -147,8 +149,8 @@ cpl_error_code cpl_array_fill_window_double_complex(cpl_array *, cpl_size, cpl_s
 cpl_error_code cpl_array_fill_window_invalid(cpl_array *, cpl_size, cpl_size);
 cpl_array *cpl_array_duplicate(const cpl_array *);
 cpl_array *cpl_array_extract(const cpl_array *, cpl_size start, cpl_size count);
-cpl_array *cpl_array_extract_real(cpl_array *);
-cpl_array *cpl_array_extract_imag(cpl_array *);
+cpl_array *cpl_array_extract_real(const cpl_array *);
+cpl_array *cpl_array_extract_imag(const cpl_array *);
 cpl_error_code cpl_array_insert_window(cpl_array *, cpl_size start, cpl_size count);
 cpl_error_code cpl_array_erase_window(cpl_array *, cpl_size start, cpl_size count);
 cpl_error_code cpl_array_insert(cpl_array *, const cpl_array *, cpl_size);
@@ -167,6 +169,7 @@ cpl_error_code cpl_array_logarithm(cpl_array *, double);
 cpl_error_code cpl_array_power(cpl_array *, double);
 cpl_error_code cpl_array_exponential(cpl_array *, double);
 #ifdef _Complex_I
+cpl_error_code cpl_array_power_complex(cpl_array *, _Complex double);
 cpl_error_code cpl_array_add_scalar_complex(cpl_array *, _Complex double);
 cpl_error_code cpl_array_subtract_scalar_complex(cpl_array *, _Complex double);
 cpl_error_code cpl_array_multiply_scalar_complex(cpl_array *, _Complex double);
@@ -186,7 +189,7 @@ cpl_error_code cpl_array_get_maxpos(const cpl_array *, cpl_size *);
 cpl_error_code cpl_array_get_minpos(const cpl_array *, cpl_size *);
 void cpl_array_dump_structure(const cpl_array *, FILE *);
 void cpl_array_dump(const cpl_array *, cpl_size start, cpl_size count, FILE *);
-cpl_array *cpl_array_cast(cpl_array *, cpl_type);
+cpl_array *cpl_array_cast(const cpl_array *, cpl_type);
 
 CPL_END_DECLS
 
