@@ -62,6 +62,9 @@ cpl_column *cpl_column_new_float_complex(cpl_size length);
 cpl_column *cpl_column_new_double_complex(cpl_size length);
 cpl_column *cpl_column_new_string(cpl_size length);
 cpl_column *cpl_column_new_array(cpl_type, cpl_size, cpl_size);
+cpl_column *cpl_column_new_complex_from_arrays(const cpl_column *,
+                                               const cpl_column *)
+    CPL_ATTR_ALLOC CPL_INTERNAL;
 
 cpl_error_code cpl_column_set_save_type(cpl_column *, cpl_type);
 cpl_type cpl_column_get_save_type(const cpl_column *);
@@ -149,16 +152,18 @@ cpl_error_code cpl_column_set_data_invalid(cpl_column *,
 cpl_error_code cpl_column_set_size(cpl_column *, cpl_size);
 cpl_error_code cpl_column_set_depth(cpl_column *, cpl_size);
 
-double cpl_column_get(cpl_column *, cpl_size, int *null); 
-_Complex double cpl_column_get_complex(cpl_column *, cpl_size , int *null);
-int cpl_column_get_int(cpl_column *, cpl_size, int *null); 
-long cpl_column_get_long(cpl_column *, cpl_size, int *null);
-long long cpl_column_get_long_long(cpl_column *, cpl_size, int *null);
-cpl_size cpl_column_get_cplsize(cpl_column *, cpl_size, int *null);
-float cpl_column_get_float(cpl_column *, cpl_size, int *null); 
-double cpl_column_get_double(cpl_column *, cpl_size, int *null); 
-_Complex float cpl_column_get_float_complex(cpl_column *, cpl_size, int *null);
-_Complex double cpl_column_get_double_complex(cpl_column *, cpl_size, int *null);
+double cpl_column_get(const cpl_column *, cpl_size, int *);
+_Complex double cpl_column_get_complex(const cpl_column *, cpl_size , int *);
+int cpl_column_get_int(const cpl_column *, cpl_size, int *); 
+long cpl_column_get_long(const cpl_column *, cpl_size, int *);
+long long cpl_column_get_long_long(const cpl_column *, cpl_size, int *);
+cpl_size cpl_column_get_cplsize(const cpl_column *, cpl_size, int *);
+float cpl_column_get_float(const cpl_column *, cpl_size, int *); 
+double cpl_column_get_double(const cpl_column *, cpl_size, int *); 
+_Complex float cpl_column_get_float_complex(const cpl_column *, cpl_size,
+                                            int *);
+_Complex double cpl_column_get_double_complex(const cpl_column *, cpl_size,
+                                              int *);
 char *cpl_column_get_string(cpl_column *, cpl_size);
 const char *cpl_column_get_string_const(const cpl_column *, cpl_size);
 cpl_array *cpl_column_get_array(cpl_column *, cpl_size);
@@ -215,10 +220,11 @@ cpl_error_code cpl_column_shift(cpl_column *, cpl_size);
 
 cpl_error_code cpl_column_set_invalid(cpl_column *, cpl_size);
 cpl_error_code cpl_column_fill_invalid(cpl_column *, cpl_size, cpl_size);
-int cpl_column_is_invalid(cpl_column *, cpl_size);
-int cpl_column_has_invalid(cpl_column *);
-int cpl_column_has_valid(cpl_column *);
+int cpl_column_is_invalid(const cpl_column *, cpl_size);
+int cpl_column_has_invalid(const cpl_column *);
+int cpl_column_has_valid(const cpl_column *);
 cpl_size cpl_column_count_invalid(cpl_column *);
+cpl_size cpl_column_count_invalid_const(const cpl_column *) CPL_INTERNAL;
 
 cpl_error_code cpl_column_fill_invalid_int(cpl_column *, int);
 cpl_error_code cpl_column_fill_invalid_long(cpl_column *, long);
@@ -235,36 +241,36 @@ cpl_error_code cpl_column_erase_segment(cpl_column *, cpl_size, cpl_size);
 cpl_error_code cpl_column_erase_pattern(cpl_column *, int *);
 cpl_error_code cpl_column_insert_segment(cpl_column *, cpl_size, cpl_size);
 
-cpl_column *cpl_column_duplicate(cpl_column *);
+cpl_column *cpl_column_duplicate(const cpl_column *);
 
 cpl_column *cpl_column_extract(cpl_column *, cpl_size, cpl_size);
 
-cpl_column *cpl_column_cast_to_int(cpl_column *);
-cpl_column *cpl_column_cast_to_long(cpl_column *);
-cpl_column *cpl_column_cast_to_long_long(cpl_column *);
-cpl_column *cpl_column_cast_to_cplsize(cpl_column *);
-cpl_column *cpl_column_cast_to_float(cpl_column *);
-cpl_column *cpl_column_cast_to_double(cpl_column *);
-cpl_column *cpl_column_cast_to_float_complex(cpl_column *);
-cpl_column *cpl_column_cast_to_double_complex(cpl_column *);
-cpl_column *cpl_column_cast_to_int_array(cpl_column *);
-cpl_column *cpl_column_cast_to_long_array(cpl_column *);
-cpl_column *cpl_column_cast_to_long_long_array(cpl_column *);
-cpl_column *cpl_column_cast_to_cplsize_array(cpl_column *);
-cpl_column *cpl_column_cast_to_float_array(cpl_column *);
-cpl_column *cpl_column_cast_to_double_array(cpl_column *);
-cpl_column *cpl_column_cast_to_float_complex_array(cpl_column *);
-cpl_column *cpl_column_cast_to_double_complex_array(cpl_column *);
-cpl_column *cpl_column_cast_to_int_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_long_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_long_long_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_cplsize_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_float_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_double_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_float_complex_flat(cpl_column *);
-cpl_column *cpl_column_cast_to_double_complex_flat(cpl_column *);
+cpl_column *cpl_column_cast_to_int(const cpl_column *);
+cpl_column *cpl_column_cast_to_long(const cpl_column *);
+cpl_column *cpl_column_cast_to_long_long(const cpl_column *);
+cpl_column *cpl_column_cast_to_cplsize(const cpl_column *);
+cpl_column *cpl_column_cast_to_float(const cpl_column *);
+cpl_column *cpl_column_cast_to_double(const cpl_column *);
+cpl_column *cpl_column_cast_to_float_complex(const cpl_column *);
+cpl_column *cpl_column_cast_to_double_complex(const cpl_column *);
+cpl_column *cpl_column_cast_to_int_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_long_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_long_long_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_cplsize_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_float_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_double_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_float_complex_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_double_complex_array(const cpl_column *);
+cpl_column *cpl_column_cast_to_int_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_long_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_long_long_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_cplsize_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_float_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_double_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_float_complex_flat(const cpl_column *);
+cpl_column *cpl_column_cast_to_double_complex_flat(const cpl_column *);
 
-cpl_error_code cpl_column_merge(cpl_column *, cpl_column *, cpl_size);
+cpl_error_code cpl_column_merge(cpl_column *, const cpl_column *, cpl_size);
 
 cpl_error_code cpl_column_add(cpl_column *, cpl_column *);
 cpl_error_code cpl_column_subtract(cpl_column *, cpl_column *);
@@ -282,22 +288,24 @@ cpl_error_code cpl_column_divide_scalar_complex(cpl_column *, _Complex double);
 cpl_error_code cpl_column_absolute(cpl_column *);
 cpl_column *cpl_column_absolute_complex(cpl_column *);
 cpl_column *cpl_column_phase_complex(cpl_column *);
-cpl_column *cpl_column_extract_real(cpl_column *);
-cpl_column *cpl_column_extract_imag(cpl_column *);
+cpl_column *cpl_column_extract_real(const cpl_column *);
+cpl_column *cpl_column_extract_imag(const cpl_column *);
 cpl_error_code cpl_column_logarithm(cpl_column *, double);
 cpl_error_code cpl_column_power(cpl_column *, double);
+cpl_error_code cpl_column_power_complex(cpl_column *,
+                                        double complex);
 cpl_error_code cpl_column_exponential(cpl_column *, double);
 cpl_error_code cpl_column_conjugate(cpl_column *);
 
-double cpl_column_get_mean(cpl_column *);
-_Complex double cpl_column_get_mean_complex(cpl_column *);
-double cpl_column_get_median(cpl_column *);
-double cpl_column_get_stdev(cpl_column *);
+double cpl_column_get_mean(const cpl_column *);
+_Complex double cpl_column_get_mean_complex(const cpl_column *);
+double cpl_column_get_median(const cpl_column *);
+double cpl_column_get_stdev(const cpl_column *);
 
-double cpl_column_get_max(cpl_column *);
-double cpl_column_get_min(cpl_column *);
-cpl_error_code cpl_column_get_maxpos(cpl_column *, cpl_size *);
-cpl_error_code cpl_column_get_minpos(cpl_column *, cpl_size *);
+double cpl_column_get_max(const cpl_column *);
+double cpl_column_get_min(const cpl_column *);
+cpl_error_code cpl_column_get_maxpos(const cpl_column *, cpl_size *);
+cpl_error_code cpl_column_get_minpos(const cpl_column *, cpl_size *);
 
 CPL_END_DECLS
 
