@@ -1,4 +1,4 @@
-/* $Id: cpl_errorstate.c,v 1.20 2012/04/26 11:57:18 llundin Exp $
+/* $Id: cpl_errorstate.c,v 1.20 2012-04-26 11:57:18 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2012/04/26 11:57:18 $
+ * $Date: 2012-04-26 11:57:18 $
  * $Revision: 1.20 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 /*-----------------------------------------------------------------------------
@@ -279,6 +279,12 @@ cpl_boolean cpl_errorstate_is_equal(cpl_errorstate self)
         to cpl_errorstate_dump() have no effect.
   @see cpl_errorstate_dump_one
 
+  CPL-based code with insufficient error checking can generate a large number
+  of CPL errors. To avoid that a subsequent call to cpl_errorstate_dump() will
+  fill up the output device, calls to the dump-function are skipped and only
+  counted and the count reported when dump_one is NULL and the CPL error code
+  is CPL_ERROR_HISTORY_LOST.
+
   Example of usage:
     @code
 
@@ -311,14 +317,14 @@ void cpl_errorstate_dump(cpl_errorstate self,
     if (!cpl_error_is_readonly()) {
 
         cpl_boolean did_call = CPL_FALSE;
+        void (*mydump_one)(unsigned, unsigned, unsigned)
+            = dump_one ? dump_one : cpl_errorstate_dump_one;
 
         /* Put CPL error system into read-only mode,
            to prevent changes by (*dump_one)() */
 
         cpl_error_set_readonly();
 
-        if (dump_one == NULL) dump_one = cpl_errorstate_dump_one;
-
         if (cpl_error_is_set()) {
             const size_t newest = estate.current;
             /* Dump all errors more recent than self */
@@ -328,6 +334,7 @@ void cpl_errorstate_dump(cpl_errorstate self,
                 /* There is at least one error to dump */
                 const size_t first = reverse ? newest : oldest;
                 const size_t last  = reverse ? oldest : newest;
+                size_t       ilost = 0;
 
                 /* The iteration through the sequence of errors
                    is done by modifying estate.current */
@@ -339,10 +346,25 @@ void cpl_errorstate_dump(cpl_errorstate self,
 
                 for (; ; reverse ? estate.current-- : estate.current++) {
                     cx_assert( estate.current > 0 );
-                    (*dump_one)((unsigned)estate.current, (unsigned)first,
-                                (unsigned)last);
+
+                    if (dump_one == NULL &&
+                        cpl_error_get_code() == CPL_ERROR_HISTORY_LOST) {
+                        ilost++;
+                    } else {
+                        if (ilost > 0) {
+                            cpl_msg_error(cpl_func, "Lost %zu CPL error(s)",
+                                          ilost);
+                            ilost = 0;
+                        }
+                        (*mydump_one)((unsigned)estate.current, (unsigned)first,
+                                      (unsigned)last);
+                    }
                     if (estate.current == last) break;
                 }
+                if (ilost > 0) {
+                    cpl_msg_error(cpl_func, "Lost %zu CPL error(s)", ilost);
+                    ilost = 0;
+                }
 
                 if (reverse) {
                     /* Reestablish the CPL error state */
@@ -358,7 +380,7 @@ void cpl_errorstate_dump(cpl_errorstate self,
             /* There are no errors more recent than self.
                Nevertheless give the dumper the opportunity to report that
                the CPL error state is empty */
-            (*dump_one)(0, 0, 0);
+            (*mydump_one)(0, 0, 0);
         }
 
         /* Put CPL error system back into normal read/write mode */
@@ -391,45 +413,6 @@ void cpl_errorstate_dump(cpl_errorstate self,
   CPL message module may be used, provided that the indentation is set back to
   its original state after the last error has been processed.
 
-  The body of cpl_errorstate_dump_one looks like this:
-    @code
-
-    const cpl_boolean is_reverse = first > last ? CPL_TRUE : CPL_FALSE;
-    const unsigned    newest     = is_reverse ? first : last;
-    const unsigned    oldest     = is_reverse ? last : first;
-    const char      * revmsg     = is_reverse ? " in reverse order" : "";
-
-
-    assert( oldest <= self );
-    assert( newest >= self );
-
-    if (newest == 0) {
-        cpl_msg_info(cpl_func, "No error(s) to dump");
-        cx_assert( oldest == 0);
-    } else {
-        cx_assert( oldest > 0);
-        cx_assert( newest >= oldest);
-        if (self == first) {
-            if (oldest == 1) {
-                cpl_msg_error(cpl_func, "Dumping all %u error(s)%s:", newest,
-                              revmsg);
-            } else {
-                cpl_msg_error(cpl_func, "Dumping the %u most recent error(s) out"
-                             " of a total of %u errors%s:", newest - oldest + 1,
-                             newest, revmsg);
-            }
-            cpl_msg_indent_more();
-        }
-
-        cpl_msg_error(cpl_func, "[%u/%u] '%s' (%u) at %s", self, newest,
-                      cpl_error_get_message(), cpl_error_get_code(),
-                      cpl_error_get_where());
-
-        if (self == last) cpl_msg_indent_less();
-    }
-
-    @endcode
-
  */
 /*----------------------------------------------------------------------------*/
 void cpl_errorstate_dump_one(unsigned self, unsigned first, unsigned last)
