@@ -188,10 +188,7 @@ static cpl_mask * cpl_mask_new_from_matrix(const cpl_matrix *, double, double);
   available. Output pixels where this is not the case are set to zero and
   flagged as rejected.
 
-  In-place filtering is not supported, but if the two images have the same
-  pixel type, then the input pixel buffer may overlap all but the 1+h first
-  rows of the output pixel buffer, where 1+2*h is the number of rows in the
-  kernel.
+  In-place filtering is not supported.
 
   Supported modes:
 
@@ -271,10 +268,6 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
                                         + (size_t)nx * (size_t)ny
                                         * cpl_type_get_sizeof
                                         (cpl_image_get_type(other)));
-    const void * psrows = (const void*)((const char*)pself
-                                        + (size_t)nsx * (size_t)(1+hsizey)
-                                        * cpl_type_get_sizeof
-                                        (cpl_image_get_type(self)));
     /* pmask may not overlap pself at all */
     const void * pmlast = (const void*)(pmask + (size_t)mx * (size_t)my);
     const void * pslast = (const void*)((const char*)pself
@@ -318,11 +311,11 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
     /* ...pother has to be above the first hsize+1 rows of pself */
     /* ... unless they have different pixel types, in which case the
            two pixel buffers may not overlap at all */
-    if (!(pself >= polast || pother >= (same_type ? psrows : pslast))) {
+    if (!(pself >= polast || pother >= pslast)) {
         return cpl_error_set_message_(CPL_ERROR_UNSUPPORTED_MODE,
                                      "pself=%p, polast=%p, pother=%p, "
-                                     "psrows=%p, pslast=%p, same_type=%d",
-                                     pself, polast, pother, psrows, pslast,
+                                     "pslast=%p, same_type=%d",
+                                     pself, polast, pother, pslast,
                                      same_type);
     }
 
@@ -854,6 +847,30 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
                     cpl_image_get_data_const(other),
                     nx, ny, hsizex, hsizey,
                     (unsigned)border);
+#ifndef CPL_MEDIAN_SAMPLE
+        if (filter == CPL_FILTER_MEDIAN && border == CPL_BORDER_FILTER) {
+            const cpl_type type = cpl_image_get_type(self);
+
+            assert(same_type);
+            assert(is_full);
+            assert(badmap == NULL);
+
+            if (type == CPL_TYPE_DOUBLE) {
+                cpl_filter_median_even_double(cpl_image_get_data(self),
+                                              cpl_image_get_data_const(other),
+                                              nx, ny, hsizex, hsizey);
+            } else if (type == CPL_TYPE_FLOAT) {
+                cpl_filter_median_even_float(cpl_image_get_data(self),
+                                             cpl_image_get_data_const(other),
+                                             nx, ny, hsizex, hsizey);
+            } else {
+                assert( type == CPL_TYPE_INT);
+                cpl_filter_median_even_int(cpl_image_get_data(self),
+                                           cpl_image_get_data_const(other),
+                                           nx, ny, hsizex, hsizey);
+            }
+        }
+#endif
     } else if (filter_full != NULL) {
         filter_full(cpl_image_get_data(self), &pbpmself,
                    cpl_image_get_data_const(other), badmap,
@@ -908,10 +925,7 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
   available. Output pixels where this is not the case are set to zero and
   flagged as bad.
 
-  In-place filtering is not supported, but if the two images have the same
-  pixel type, then the input pixel buffer may overlap all but the 1+H first
-  rows of the output pixel buffer, where 1+2*H is the number of rows in the
-  kernel.
+  In-place filtering is not supported.
 
   Supported filters: @c CPL_FILTER_LINEAR, @c CPL_FILTER_MORPHO,
                      @c CPL_FILTER_LINEAR_SCALE, @c CPL_FILTER_MORPHO_SCALE.
@@ -972,10 +986,6 @@ cpl_error_code cpl_image_filter(cpl_image        * self,
                                         + (size_t)nx * (size_t)ny
                                         * cpl_type_get_sizeof
                                         (cpl_image_get_type(other)));
-    const void * psrows = (const void*)((const char*)pself
-                                        + (size_t)nsx * (size_t)(1+hsizey)
-                                        * cpl_type_get_sizeof
-                                        (cpl_image_get_type(self)));
     /* pkernel may not overlap pself at all */
     const void * pmlast = (const void*)(pkernel + mx * my);
     const void * pslast = (const void*)((const char*)pself
@@ -989,8 +999,6 @@ cpl_error_code cpl_image_filter(cpl_image        * self,
                                       const double *, cpl_size, cpl_size,
                                       cpl_boolean,
                                       cpl_size, cpl_size, cpl_border_mode);
-    cpl_boolean  same_type = cpl_image_get_type(self)
-        == cpl_image_get_type(other);
     cpl_boolean dodiv = CPL_FALSE;
     cpl_size i;
 
@@ -1002,7 +1010,7 @@ cpl_error_code cpl_image_filter(cpl_image        * self,
     /* ...pother has to be above the first hsize+1 rows of pself */
     /* ... unless they have different pixel types, in which case the
            two pixel buffers may not overlap at all */
-    cpl_ensure_code(pself >= polast || pother >= (same_type ? psrows : pslast),
+    cpl_ensure_code(pself >= polast || pother >= pslast,
         CPL_ERROR_UNSUPPORTED_MODE);
 
     /* If this check fails, the caller is doing something really weird... */
