@@ -33,15 +33,9 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <math.h>
-#include <float.h>
-
 #include "cpl_memory.h"
-#include "cpl_stats.h"
+#include "cpl_stats_impl.h"
+#include "cpl_error_impl.h"
 #include "cpl_image_bpm.h"
 #include "cpl_image_stats.h"
 #include "cpl_mask.h"
@@ -49,6 +43,13 @@
 
 #include "cpl_image_defs.h"
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <math.h>
+#include <float.h>
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -60,18 +61,19 @@
 /*----------------------------------------------------------------------------*/
 #define CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(op, OP)                  \
                                                                         \
-     cpl_stats * stats =                                                \
-        cpl_stats_new_from_image_window(image,                          \
-                                        CPL_CONCAT2X(CPL_STATS, OP),    \
-                                        llx, lly,  urx, ury);           \
-    double res;                                                         \
+    cpl_stats stats;                                                    \
+    const cpl_error_code icode =                                        \
+        cpl_stats_fill_from_image_window(&stats, image,                 \
+                                         CPL_CONCAT2X(CPL_STATS, OP),   \
+                                         llx, lly,  urx, ury);          \
+    double res = 0.0;                                                   \
                                                                         \
-    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               \
-                                                                        \
-    /* Should not be able to fail now */                                \
-    res = CPL_CONCAT2X(cpl_stats_get, op)((const cpl_stats *)stats);    \
-                                                                        \
-    cpl_stats_delete(stats);                                            \
+    if (icode) {                                                        \
+        (void)cpl_error_set_where_();                                   \
+    } else {                                                            \
+        /* Should not be able to fail now */                            \
+        res = CPL_CONCAT2X(cpl_stats_get, op)((const cpl_stats *)&stats); \
+    }                                                                   \
                                                                         \
     return res                                                     
 
@@ -86,16 +88,19 @@
 /*----------------------------------------------------------------------------*/
 #define CPL_IMAGE_GET_STATS_DOUBLE_BODY(op, OP)                         \
                                                                         \
-    cpl_stats * stats =                                                 \
-        cpl_stats_new_from_image(image, CPL_CONCAT2X(CPL_STATS, OP));   \
-    double res;                                                         \
+    cpl_stats stats;                                                    \
+    const cpl_error_code icode =                                        \
+        cpl_stats_fill_from_image(&stats, image, CPL_CONCAT2X(CPL_STATS, OP)); \
+    double res = 0.0;                                                   \
                                                                         \
-    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               \
                                                                         \
-    /* Should not be able to fail now */                                \
-    res = CPL_CONCAT2X(cpl_stats_get, op)((const cpl_stats *)stats);    \
+    if (icode) {                                                        \
+        (void)cpl_error_set_where_();                                   \
+    } else {                                                            \
+        /* Should not be able to fail now */                            \
+        res = CPL_CONCAT2X(cpl_stats_get, op)((const cpl_stats *)&stats); \
+    }                                                                   \
                                                                         \
-    cpl_stats_delete(stats);                                            \
                                                                         \
     return res                                                     
 
@@ -536,22 +541,20 @@ cpl_error_code cpl_image_get_minpos_window(const cpl_image * image,
                                            cpl_size        * px,
                                            cpl_size        * py)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
 
     cpl_ensure_code(px != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(py != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image_window(image, CPL_STATS_MINPOS,
-                                            llx, lly,  urx, ury);
-
-    cpl_ensure_code(stats != NULL, cpl_error_get_code());
+    if (cpl_stats_fill_from_image_window(&stats, image, CPL_STATS_MINPOS,
+                                         llx, lly,  urx, ury)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    *px = cpl_stats_get_min_x(stats);
-    *py = cpl_stats_get_min_y(stats);
-
-    cpl_stats_delete(stats);
+    *px = cpl_stats_get_min_x(&stats);
+    *py = cpl_stats_get_min_y(&stats);
 
     return CPL_ERROR_NONE;
 }
@@ -577,21 +580,19 @@ cpl_error_code cpl_image_get_minpos(const cpl_image * image,
                                     cpl_size        * py)
 {
 
-    cpl_stats * stats;
+    cpl_stats stats;
 
     cpl_ensure_code(px != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(py != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image(image, CPL_STATS_MINPOS);
-
-    cpl_ensure_code(stats != NULL, cpl_error_get_code());
-                                                                        
+    if (cpl_stats_fill_from_image(&stats, image, CPL_STATS_MINPOS)) {
+        return cpl_error_set_where_();
+    }
+                                                                       
     /* Should not be able to fail now */
 
-    *px = cpl_stats_get_min_x(stats);
-    *py = cpl_stats_get_min_y(stats);
-
-    cpl_stats_delete(stats);
+    *px = cpl_stats_get_min_x(&stats);
+    *py = cpl_stats_get_min_y(&stats);
 
     return CPL_ERROR_NONE;
 
@@ -620,22 +621,20 @@ cpl_error_code cpl_image_get_maxpos_window(const cpl_image * image,
                                            cpl_size        * px,
                                            cpl_size        * py)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
 
     cpl_ensure_code(px != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(py != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image_window(image, CPL_STATS_MAXPOS,
-                                            llx, lly,  urx, ury);
-
-    cpl_ensure_code(stats != NULL, cpl_error_get_code());
+    if (cpl_stats_fill_from_image_window(&stats, image, CPL_STATS_MAXPOS,
+                                            llx, lly,  urx, ury)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    *px = cpl_stats_get_max_x(stats);
-    *py = cpl_stats_get_max_y(stats);
-
-    cpl_stats_delete(stats);
+    *px = cpl_stats_get_max_x(&stats);
+    *py = cpl_stats_get_max_y(&stats);
 
     return CPL_ERROR_NONE;
 }
@@ -655,21 +654,19 @@ cpl_error_code cpl_image_get_maxpos(const cpl_image * image,
                                     cpl_size        * px,
                                     cpl_size        * py)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
 
     cpl_ensure_code(px != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(py != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image(image, CPL_STATS_MAXPOS);
-
-    cpl_ensure_code(stats != NULL, cpl_error_get_code());
+    if (cpl_stats_fill_from_image(&stats, image, CPL_STATS_MAXPOS)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    *px = cpl_stats_get_max_x(stats);
-    *py = cpl_stats_get_max_y(stats);
-
-    cpl_stats_delete(stats);
+    *px = cpl_stats_get_max_x(&stats);
+    *py = cpl_stats_get_max_y(&stats);
 
     return CPL_ERROR_NONE;
 }
@@ -712,23 +709,21 @@ double cpl_image_get_median_dev_window(const cpl_image * image,
                                        cpl_size          ury,
                                        double          * sigma)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
     double median;
 
 
     cpl_ensure_code(sigma != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image_window(image, CPL_STATS_MEDIAN_DEV,
-                                            llx, lly,  urx, ury);
-
-    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               
+    if (cpl_stats_fill_from_image_window(&stats, image, CPL_STATS_MEDIAN_DEV,
+                                            llx, lly,  urx, ury)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    median = cpl_stats_get_median(stats);
-    *sigma = cpl_stats_get_median_dev(stats);
-
-    cpl_stats_delete(stats);
+    median = cpl_stats_get_median(&stats);
+    *sigma = cpl_stats_get_median_dev(&stats);
 
     return median;
 }
@@ -751,22 +746,20 @@ double cpl_image_get_median_dev_window(const cpl_image * image,
 double cpl_image_get_median_dev(const cpl_image * image,
                                 double          * sigma)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
     double median;
 
 
     cpl_ensure_code(sigma != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image(image, CPL_STATS_MEDIAN_DEV);
-
-    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               
+    if (cpl_stats_fill_from_image(&stats, image, CPL_STATS_MEDIAN_DEV)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    median = cpl_stats_get_median(stats);
-    *sigma = cpl_stats_get_median_dev(stats);
-
-    cpl_stats_delete(stats);
+    median = cpl_stats_get_median(&stats);
+    *sigma = cpl_stats_get_median_dev(&stats);
 
     return median;
 }
@@ -811,23 +804,21 @@ double cpl_image_get_mad_window(const cpl_image * image,
                                 cpl_size          ury,
                                 double          * sigma)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
     double median;
 
 
     cpl_ensure_code(sigma != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image_window(image, CPL_STATS_MAD,
-                                            llx, lly,  urx, ury);
-
-    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               
+    if (cpl_stats_fill_from_image_window(&stats, image, CPL_STATS_MAD,
+                                            llx, lly,  urx, ury)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    median = cpl_stats_get_median(stats);
-    *sigma = cpl_stats_get_mad(stats);
-
-    cpl_stats_delete(stats);
+    median = cpl_stats_get_median(&stats);
+    *sigma = cpl_stats_get_mad(&stats);
 
     return median;
 }
@@ -850,22 +841,20 @@ double cpl_image_get_mad_window(const cpl_image * image,
 double cpl_image_get_mad(const cpl_image * image,
                          double          * sigma)
 {
-    cpl_stats * stats;
+    cpl_stats stats;
     double median;
 
 
     cpl_ensure_code(sigma != NULL, CPL_ERROR_NULL_INPUT);
 
-    stats = cpl_stats_new_from_image(image, CPL_STATS_MAD);
-
-    cpl_ensure(stats != NULL, cpl_error_get_code(), 0.0);               
+    if (cpl_stats_fill_from_image(&stats, image, CPL_STATS_MAD)) {
+        return cpl_error_set_where_();
+    }
                                                                         
     /* Should not be able to fail now */
 
-    median = cpl_stats_get_median(stats);
-    *sigma = cpl_stats_get_mad(stats);
-
-    cpl_stats_delete(stats);
+    median = cpl_stats_get_median(&stats);
+    *sigma = cpl_stats_get_mad(&stats);
 
     return median;
 }
