@@ -283,11 +283,24 @@ const char * cpl_get_description(unsigned self)
 #endif
 
 #if defined CPL_FFTW_INSTALLED && defined CPL_FFTWF_INSTALLED
-#define CPL_FFTW_APPEND ", FFTW (double and single precision)"
+#if defined CPL_FFTW_VERSION && defined CPL_FFTWF_VERSION
+#define CPL_FFTW_APPEND ", FFTW (normal precision) = " CPL_FFTW_VERSION \
+                        ", FFTW (single precision) = " CPL_FFTWF_VERSION
+#else
+#define CPL_FFTW_APPEND ", FFTW (normal and single precision)"
+#endif
 #elif defined CPL_FFTW_INSTALLED
-#define CPL_FFTW_APPEND ", FFTW (double precision)"
+#if defined CPL_FFTW_VERSION
+#define CPL_FFTW_APPEND ", FFTW (normal precision) = " CPL_FFTW_VERSION
+#else
+#define CPL_FFTW_APPEND ", FFTW (normal precision)"
+#endif
 #elif defined CPL_FFTWF_INSTALLED
+#if defined CPL_FFTWF_VERSION
+#define CPL_FFTW_APPEND ", FFTW (single precision) = " CPL_FFTWF_VERSION
+#else
 #define CPL_FFTW_APPEND ", FFTW (single precision)"
+#endif
 #else
 #define CPL_FFTW_APPEND " (FFTW unavailable)"
 #endif
