@@ -44,6 +44,20 @@
 #define CPL_BEGIN_DECLS  CX_BEGIN_DECLS
 #define CPL_END_DECLS    CX_END_DECLS
 
+
+/*
+ * Macro to silence compiler warnings on unused arguments
+ * and variables. Active by default, but can be turned off
+ * for debug builds using CPL_WITH_UNUSED_WARNINGS.
+ */
+
+#ifdef CPL_UNUSED_WARNINGS
+#  define CPL_UNUSED(arg)
+#else
+#  define CPL_UNUSED(arg) (void)(arg)
+#endif
+
+
 /* Needed to concatenate two and three macro arguments */
 #define CPL_CONCAT(a,b) a ## _ ## b
 #define CPL_CONCAT2X(a,b) CPL_CONCAT(a,b)
