@@ -151,6 +151,12 @@ cpl_error_code cpl_matrix_power(cpl_matrix *, double);
 cpl_matrix *cpl_matrix_product_create(const cpl_matrix *,
                                       const cpl_matrix *) CPL_ATTR_ALLOC;
 cpl_matrix *cpl_matrix_transpose_create(const cpl_matrix *) CPL_ATTR_ALLOC;
+cpl_error_code cpl_matrix_product_normal(cpl_matrix *,
+                                         const cpl_matrix *);
+
+cpl_error_code cpl_matrix_product_transpose(cpl_matrix *,
+                                            const cpl_matrix *,
+                                            const cpl_matrix *);
 
 /*
  * More complex operations
