@@ -32,9 +32,6 @@
  
 CPL_BEGIN_DECLS
 
-cpl_error_code cpl_matrix_product_normal(cpl_matrix *,
-                                         const cpl_matrix *);
-
 cpl_matrix * cpl_matrix_product_normal_create(const cpl_matrix *)
     CPL_ATTR_ALLOC;
 
@@ -43,10 +40,6 @@ cpl_error_code cpl_matrix_solve_spd(cpl_matrix *, cpl_matrix *);
 cpl_error_code cpl_matrix_product(cpl_matrix *, const cpl_matrix *,
                                   const cpl_matrix *);
 
-cpl_error_code cpl_matrix_product_transpose(cpl_matrix *,
-                                            const cpl_matrix *,
-                                            const cpl_matrix *);
-
 cpl_error_code cpl_matrix_product_bilinear(cpl_matrix *,
                                            const cpl_matrix *,
                                            const cpl_matrix *);
