@@ -323,7 +323,7 @@ char * cpl_vsprintf(const char * format, va_list arglist)
         cx_free(self);
         self = NULL;
         (void)cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT, "cx_vasprintf() "
-                                     "failed with %d on format=%s", nlen,
+                                     "returned %d < 0 on format=%s", nlen,
                                      format);
     } else if (self == NULL) {
         (void)cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT, "cx_vasprintf() "
