@@ -300,7 +300,7 @@ static void _cpl_change_width(int i)
 
     struct winsize win;
 
-    (void) i;  /* To avoid compiler warning */
+    CPL_UNUSED(i);
 
     if (ioctl(out_stream, TIOCGWINSZ, &win) < 0 || win.ws_col < 1)
         page_width = DEFAULT_WIDTH;
