@@ -42,7 +42,7 @@
 #include <assert.h>
 
 #include "cpl_memory.h"
-#include "cpl_stats.h"
+#include "cpl_stats_impl.h"
 #include "cpl_image_bpm.h"
 #include "cpl_image_stats.h"
 #include "cpl_mask.h"
@@ -102,31 +102,6 @@
 /*----------------------------------------------------------------------------*/
 /**@{*/
 
-/*-----------------------------------------------------------------------------
-                            Type definition
- -----------------------------------------------------------------------------*/
-struct _cpl_stats_
-{
-    double      min;
-    double      max;
-    double      mean;
-    double      med;
-    double      med_dev;
-    double      mad;
-    double      stdev;
-    double      flux;
-    double      absflux;
-    double      sqflux;
-    double      centroid_x;
-    double      centroid_y;
-    cpl_size    min_x;
-    cpl_size    min_y;
-    cpl_size    max_x;
-    cpl_size    max_y;
-    cpl_size    npix;
-    cpl_stats_mode mode;
-};
-
 /*-----------------------------------------------------------------------------
                             Function codes
  -----------------------------------------------------------------------------*/
@@ -504,12 +479,71 @@ cpl_stats * cpl_stats_new_from_image_window(
         cpl_size            lly,
         cpl_size            urx,
         cpl_size            ury)
+{
+    /* Allocate stat object */
+    cpl_stats  * self = cpl_malloc(sizeof(cpl_stats));
+    const cpl_error_code code =
+        cpl_stats_fill_from_image_window(self, image, mode, llx, lly, urx, ury);
+
+    if (code) {
+        (void)cpl_error_set_where_();
+        cpl_free(self);
+        self = NULL;
+    }
+
+    return self;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Compute various statistics of an image sub-window.
+  @param    self        Pre-allocated struct to fill
+  @param    image       Input image
+  @param    mode        Bit field specifying which statistics to compute
+  @return   CPL_ERROR_NONE or the relevant the #_cpl_error_code_
+  @see cpl_stats_new_from_image_window()
+ */
+/*----------------------------------------------------------------------------*/
+cpl_error_code cpl_stats_fill_from_image(cpl_stats * self,
+                                         const cpl_image *   image,
+                                         cpl_stats_mode      mode)
+{
+    const cpl_error_code code =
+        cpl_stats_fill_from_image_window(self, image, mode, 1, 1,
+                                         cpl_image_get_size_x(image),
+                                         cpl_image_get_size_y(image));
+
+    return code ? cpl_error_set_where_() : CPL_ERROR_NONE;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Compute various statistics of an image sub-window.
+  @param    self        Pre-allocated struct to fill
+  @param    image       Input image.
+  @param    mode        Bit field specifying which statistics to compute
+  @param    llx         Lower left x position (FITS convention)
+  @param    lly         Lower left y position (FITS convention)
+  @param    urx         Upper right x position (FITS convention)
+  @param    ury         Upper right y position (FITS convention)
+  @return   CPL_ERROR_NONE or the relevant the #_cpl_error_code_
+  @see cpl_stats_new_from_image_window()
+ */
+/*----------------------------------------------------------------------------*/
+cpl_error_code cpl_stats_fill_from_image_window(cpl_stats * self,
+                                                const cpl_image *   image,
+                                                cpl_stats_mode      mode,
+                                                cpl_size            llx,
+                                                cpl_size            lly,
+                                                cpl_size            urx,
+                                                cpl_size            ury)
 {
     const size_t llxsz = (size_t)llx;
     const size_t llysz = (size_t)lly;
     const size_t urxsz = (size_t)urx;
     const size_t urysz = (size_t)ury;
-    cpl_stats    * self;
     double         pix_sum = 0.0;
     double         sqr_sum = 0.0;
     double         abs_sum = 0.0;
@@ -524,6 +558,7 @@ cpl_stats * cpl_stats_new_from_image_window(
     cpl_size       npix;
     const cpl_size mpix = (urx-llx+1) * (ury-lly+1);
     cpl_size       pos;
+    cpl_ifalloc    copybuf;
     /* Two statistics computation categories defined here */
     const cpl_stats_mode minmax_cat = mode &
         (CPL_STATS_MIN|CPL_STATS_MAX|CPL_STATS_MINPOS|
@@ -542,19 +577,18 @@ cpl_stats * cpl_stats_new_from_image_window(
     size_t              i, j;
 
     /* Test inputs */
-    cpl_ensure(image != NULL,    CPL_ERROR_NULL_INPUT,          NULL);
+    cpl_ensure_code(image != NULL,    CPL_ERROR_NULL_INPUT);
 
-    cpl_ensure(llx >  0,         CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
-    cpl_ensure(lly >  0,         CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
-    cpl_ensure(urx <= image->nx, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
-    cpl_ensure(ury <= image->ny, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
-    cpl_ensure(llx <= urx,       CPL_ERROR_ILLEGAL_INPUT,       NULL);
-    cpl_ensure(lly <= ury,       CPL_ERROR_ILLEGAL_INPUT,       NULL);
+    cpl_ensure_code(llx >  0,         CPL_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_ensure_code(lly >  0,         CPL_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_ensure_code(urx <= image->nx, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_ensure_code(ury <= image->ny, CPL_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_ensure_code(llx <= urx,       CPL_ERROR_ILLEGAL_INPUT);
+    cpl_ensure_code(lly <= ury,       CPL_ERROR_ILLEGAL_INPUT);
 
-    cpl_ensure(mode != 1, CPL_ERROR_INVALID_TYPE, NULL);
-    cpl_ensure(mode != 0, CPL_ERROR_UNSUPPORTED_MODE, NULL);
-    cpl_ensure((mode & ~CPL_STATS_ALL) == 0, CPL_ERROR_UNSUPPORTED_MODE,
-               NULL);
+    cpl_ensure_code(mode != 1, CPL_ERROR_INVALID_TYPE);
+    cpl_ensure_code(mode != 0, CPL_ERROR_UNSUPPORTED_MODE);
+    cpl_ensure_code((mode & ~CPL_STATS_ALL) == 0, CPL_ERROR_UNSUPPORTED_MODE);
 
     /* Need the median for the mean and median median absolute deviation */
     if (mode & (CPL_STATS_MEDIAN_DEV | CPL_STATS_MAD))
@@ -580,16 +614,13 @@ cpl_stats * cpl_stats_new_from_image_window(
         }
 
         /* Verify that there are good pixels */
-        cpl_ensure(firstgoodpos >= 0, CPL_ERROR_DATA_NOT_FOUND, NULL);
+        cpl_ensure_code(firstgoodpos >= 0, CPL_ERROR_DATA_NOT_FOUND);
 
     } else {
         firstgoodpos = (llx-1)+(lly-1)*image->nx;
     }
     npix = mpix - nbadpix;
 
-    /* Allocate stat object */
-    self = cpl_malloc(sizeof(cpl_stats));
-
     /* When a member of the struct is accessed check that it was initialized */
     self->mode = mode;
 
@@ -599,6 +630,9 @@ cpl_stats * cpl_stats_new_from_image_window(
     /* The tests should stay outside the loops */
 
     if (mode & CPL_STATS_MEDIAN) {
+        /* Duplicate the pixels inside the window - maybe reused below */
+        cpl_ifalloc_set(&copybuf,
+                        (size_t)npix * cpl_type_get_sizeof(image->type));
         switch (image->type) {
 #define CPL_OPERATION CPL_IMAGE_STATS_MEDIAN
 #define CPL_CLASS CPL_CLASS_DOUBLE
@@ -613,10 +647,11 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
         default:
             /* See comment in previous switch() default: */
-            cpl_stats_delete(self);
-            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-            return NULL;
+            cpl_ifalloc_free(&copybuf);
+            return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
         }
+    } else {
+        cpl_ifalloc_set(&copybuf, 0); /* Initialize for deallocation */
     }
 
     if (mode == CPL_STATS_ALL) {
@@ -643,9 +678,8 @@ cpl_stats * cpl_stats_new_from_image_window(
          * in this function. For that case, we keep setting the appropriate
          * error code in this default section.
          */
-            cpl_stats_delete(self);
-            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-            return NULL;
+            cpl_ifalloc_free(&copybuf);
+            return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
         }
 
     } else {
@@ -665,9 +699,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
             default:
                 /* See comment in previous switch() default: */
-                cpl_stats_delete(self);
-                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-                return NULL;
+                cpl_ifalloc_free(&copybuf);
+                return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
             }
 
         }
@@ -687,9 +720,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
             default:
                 /* See comment in previous switch() default: */
-                cpl_stats_delete(self);
-                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-                return NULL;
+                cpl_ifalloc_free(&copybuf);
+                return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
             }
         }
         if (variance_cat) {
@@ -708,9 +740,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
             default:
                 /* See comment in previous switch() default: */
-                cpl_stats_delete(self);
-                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-                return NULL;
+                cpl_ifalloc_free(&copybuf);
+                return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
             }
         }
 
@@ -729,9 +760,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
             default:
                 /* See comment in previous switch() default: */
-                cpl_stats_delete(self);
-                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-                return NULL;
+                cpl_ifalloc_free(&copybuf);
+                return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
             }
         }
     }
@@ -750,9 +780,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
         default:
             /* See comment in previous switch() default: */
-            cpl_stats_delete(self);
-            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-            return NULL;
+            cpl_ifalloc_free(&copybuf);
+            return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
         }
     }
 
@@ -787,9 +816,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_OPERATION
           default:
               /* See comment in previous switch() default: */
-              cpl_stats_delete(self);
-              (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
-              return NULL;
+              cpl_ifalloc_free(&copybuf);
+              return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
         }
         if (sum_z > 0.0) {
             self->centroid_x =  sum_xz / sum_z;
@@ -834,7 +862,8 @@ cpl_stats * cpl_stats_new_from_image_window(
 
     if (mode & CPL_STATS_MEDIAN_DEV) self->med_dev = dev_sum / (double) npix;
 
-    return self;
+    cpl_ifalloc_free(&copybuf);
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
