@@ -185,15 +185,13 @@
 
     case CPL_TYPE_T:
     {
-        /* Duplicate the pixels inside the window */
-        CPL_TYPE * copybuf = cpl_malloc((size_t)npix * sizeof(CPL_TYPE));
-
+        CPL_TYPE * copybuft = (CPL_TYPE *)cpl_ifalloc_get(&copybuf);
         if (nbadpix == 0) {
 
             /* All pixels are good */
 
             /* Cannot fail here */
-            (void)cpl_tools_copy_window((void*)copybuf,
+            (void)cpl_tools_copy_window((void*)copybuft,
                                         image->pixels,
                                         sizeof(CPL_TYPE),
                                         image->nx, image->ny,
@@ -212,7 +210,7 @@
                 for (i = llxsz - 1; i < urxsz; i++) {
                     /* Take only good pixels */
                     if (pbpm[i] == CPL_BINARY_0) {
-                        copybuf[ngood++] = pi[i];
+                        copybuft[ngood++] = pi[i];
                     }
                 }
             }
@@ -220,10 +218,8 @@
         }
 
         /* Compute the median */
-        self->med = CPL_TYPE_ADD(cpl_tools_get_median)(copybuf, npix);
+        self->med = CPL_TYPE_ADD(cpl_tools_get_median)(copybuft, npix);
 
-        cpl_free(copybuf);
-    
         break;
     }
 
@@ -275,9 +271,7 @@
 
   case CPL_TYPE_T:
   {
-
-      /* Duplicate the pixels inside the window */
-      CPL_TYPE * copybuf = cpl_malloc((size_t)npix * sizeof(CPL_TYPE));
+      CPL_TYPE * copybuft = (CPL_TYPE *)cpl_ifalloc_get(&copybuf);
       cpl_size   ngood   = 0;
 
 
@@ -295,7 +289,7 @@
                 for (i = llxsz - 1; i < urxsz; i++) {
                     /* Take only good pixels */
                     if (pbpm[i] == CPL_BINARY_0) {
-                        copybuf[ngood++] = fabs((double)pi[i]-self->med);
+                        copybuft[ngood++] = fabs((double)pi[i]-self->med);
                     }
                 }
             }
@@ -306,7 +300,7 @@
 
             for (j = llysz - 1; j < urysz; j++, pi += image->nx) {
                 for (i = llxsz - 1; i < urxsz; i++) {
-                    copybuf[ngood++] = fabs((double)pi[i]-self->med);
+                    copybuft[ngood++] = fabs((double)pi[i]-self->med);
                 }
             }
       }
@@ -315,9 +309,7 @@
       cpl_tools_add_flops( 2 * npix );
 
       /* Compute the median */
-      self->mad = CPL_TYPE_ADD(cpl_tools_get_median)(copybuf, npix);
-
-      cpl_free(copybuf);
+      self->mad = CPL_TYPE_ADD(cpl_tools_get_median)(copybuft, npix);
 
       break;
   }
