@@ -238,6 +238,74 @@ static cpl_flops flop_count = 0;
 #undef CPL_TOOLS_SORT_LT
 
 
+/*----------------------------------------------------------------------------*/
+/**
+   @internal
+   @brief Initializer with conditional allocation for ifalloc variable
+   @param self  The ifalloc variable to be conditionally allocated
+   @param size  The number of bytes needed, zero guarantees no heap allocation
+   @see cpl_ifalloc_free
+   @note Calling this function with a pointer that has previously been passed
+         to this function with a non-zero size can lead to a memory leak.
+
+   Example of usage:
+    @code
+
+    cpl_ifalloc mybuf;
+    cpl_ifalloc_set(&mybuf, 0);
+
+    if (!myerror) {
+      double * mynumbers;
+
+      cpl_ifalloc_set(&mybuf, mysize * sizeof(*mynumbers));
+
+      mynumbers = (double*)cpl_ifalloc_get(&mybuf);
+    }
+
+    cpl_ifalloc_free(&mybuf);
+
+    @endcode
+
+ */
+/*----------------------------------------------------------------------------*/
+inline void cpl_ifalloc_set(cpl_ifalloc * self, size_t size)
+{
+    if (size > (size_t)CPL_IFALLOC_SZ) {
+        self->data = self->heap = cpl_malloc(size);
+    } else {
+        self->data = (void*)(self->stack);
+        self->heap = NULL;
+    }
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+   @internal
+   @brief Get pointer to data area of ifalloc variable
+   @param self  The ifalloc variable to accessed
+   @see cpl_ifalloc_set
+   @note The returned pointer becomes stale after a call to cpl_ifalloc_free()
+
+ */
+/*----------------------------------------------------------------------------*/
+inline void * cpl_ifalloc_get(cpl_ifalloc * self)
+{
+    return self->data;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+   @internal
+   @brief Free all memory associated with ifalloc variable
+   @param self  The ifalloc variable to be deallocated
+   @see cpl_ifalloc_set
+ */
+/*----------------------------------------------------------------------------*/
+inline void cpl_ifalloc_free(cpl_ifalloc * self)
+{
+    cpl_free(self->heap);
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
