@@ -149,9 +149,39 @@ typedef double cpl_long_double;
 #define CPL_LDBL_EPSILON_STR "DBL_EPSILON"
 #endif
 
+/*----------------------------------------------------------------------------*/
+/**
+   @hideinitializer
+   @ingroup cpl_tools
+   @internal
+   @brief Maximum size [byte] of ifalloc variable without heap allocation
+   @see cpl_ifalloc_set
+ */
+/*----------------------------------------------------------------------------*/
+#ifndef CPL_IFALLOC_SZ
+#define CPL_IFALLOC_SZ 1024
+#endif
+
+/*----------------------------------------------------------------------------*/
+/**
+   @ingroup cpl_tools
+   @internal
+   @brief The struct used for the ifalloc variable
+   @see cpl_ifalloc_set
+ */
+/*----------------------------------------------------------------------------*/
+typedef struct {
+    void * data;
+    void * heap;
+    char stack[CPL_IFALLOC_SZ];
+} cpl_ifalloc;
+
 /*-----------------------------------------------------------------------------
                             Function prototypes
  -----------------------------------------------------------------------------*/
+void cpl_ifalloc_set(cpl_ifalloc *, size_t) CPL_INTERNAL CPL_ATTR_NONNULL;
+void * cpl_ifalloc_get(cpl_ifalloc *) CPL_INTERNAL CPL_ATTR_NONNULL;
+void cpl_ifalloc_free(cpl_ifalloc *) CPL_INTERNAL CPL_ATTR_NONNULL;
 
 /* Operations on arrays */
 double cpl_tools_get_kth_double(double *, cpl_size, cpl_size) CPL_ATTR_NONNULL;
