@@ -136,6 +136,10 @@ cpl_type_get_sizeof(cpl_type type)
             sz = sizeof(long long int);
             break;
 
+        case CPL_TYPE_SIZE:
+            sz = sizeof(cpl_size);
+            break;
+
         case CPL_TYPE_FLOAT:
             sz = sizeof(float);
             break;
@@ -228,6 +232,10 @@ cpl_type_get_name(cpl_type type)
             self = "long long int array";
             break;
 
+        case CPL_TYPE_SIZE:
+            self = "cpl size array";
+            break;
+
         case CPL_TYPE_FLOAT:
             self = "float array";
             break;
@@ -308,6 +316,10 @@ cpl_type_get_name(cpl_type type)
             self = "long long int";
             break;
 
+        case CPL_TYPE_SIZE:
+            self = "cpl size";
+            break;
+
         case CPL_TYPE_FLOAT:
             self = "float";
             break;
