@@ -75,6 +75,7 @@ static void cpl_image_divide_test_one(void);
 static void cpl_image_power_test(void);
 static void cpl_image_hypot_test(void);
 static void cpl_image_bitwise_test(cpl_size, cpl_size);
+static void cpl_image_divide_create_test(void);
 
 /*-----------------------------------------------------------------------------
                                   Main
@@ -163,6 +164,8 @@ int main(void)
     cpl_image_copy_test();
     cpl_image_turn_flip_test();
 
+    cpl_image_divide_create_test();
+
     /* Tests of cpl_image_collapse_window_create() */
     /* Tests of cpl_image_collapse_create() */
 
@@ -2179,3 +2182,125 @@ static void cpl_image_bitwise_test(cpl_size nx, cpl_size ny)
     cpl_image_delete(im2);
     cpl_mask_delete(mask0);
 }
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Test the cpl_image_divide_create function
+  @return   void
+  This test checks the behaviour of the cpl_image_divide_create function for
+  various cases of division by zeros and bad pixel masks.
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_divide_create_test(void)
+{
+    cpl_image * a = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
+    cpl_image * b = cpl_image_new(3, 3, CPL_TYPE_FLOAT);
+    cpl_image * c = NULL;
+    const cpl_mask * m = NULL;
+    float * data_a = cpl_image_get_data_float(a);
+    float * data_b = cpl_image_get_data_float(b);
+    int i;
+
+    /* Make sure the reference images are initialised. Otherwise stop here. */
+    cpl_test_nonnull(a);
+    cpl_test_nonnull(b);
+    cpl_test_nonnull(data_a);
+    cpl_test_nonnull(data_b);
+    if (a == NULL || b == NULL || data_a == NULL || data_b == NULL) return;
+
+    /*                [ 6 -7  8]
+     * Set image A to [-3  4 -5]
+     *                [ 0 -1  2]
+     */
+    for (i = 0; i < 9; ++i) data_a[i] = (float)i * (i % 2 == 0 ? 1.0 : -1.0);
+
+    /* Image B should be all zeros from the function call cpl_image_new. */
+    for (i = 0; i < 9; ++i) cpl_test(data_b[i] == 0.0);
+
+    /* Check that dividing by image B with all zeros produces a CPL divide by
+     * zero error code. */
+    c = cpl_image_divide_create(a, b);
+    cpl_test_null(c);
+    cpl_test_error(CPL_ERROR_DIVISION_BY_ZERO);
+
+    /* Check that the following holds A/B = C, where
+     *     [ 6 -7  8]       1   [6 0 0]           [ 1  0  0]
+     * A = [-3  4 -5]  B = -- * [0 4 5]  C = 10 * [ 0  1 -1]
+     *     [ 0 -1  2]      10   [0 1 0]           [ 0 -1  0]
+     * The bad pixel masks should be the following (X indicates bad, '.' good):
+     *         [. X X]
+     * C_bpm = [X . .]
+     *         [X . X]
+     */
+    data_b[6] = 0.6; data_b[7] = 0.0; data_b[8] = 0.0;
+    data_b[3] = 0.0; data_b[4] = 0.4; data_b[5] = 0.5;
+    data_b[0] = 0.0; data_b[1] = 0.1; data_b[2] = 0.0;
+    c = cpl_image_divide_create(a, b);
+    cpl_test_nonnull(c);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[0],   0.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[1], -10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[2],   0.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[3],   0.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[4],  10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[5], -10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[6],  10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[7],   0.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[8],   0.0, FLT_EPSILON);
+    /* Check the bad pixels. */
+    m = cpl_image_get_bpm_const(c);
+    cpl_test_nonnull(m);
+    cpl_test(cpl_mask_get_data_const(m)[0] == CPL_BINARY_1);
+    cpl_test(cpl_mask_get_data_const(m)[1] == CPL_BINARY_0);
+    cpl_test(cpl_mask_get_data_const(m)[2] == CPL_BINARY_1);
+    cpl_test(cpl_mask_get_data_const(m)[3] == CPL_BINARY_1);
+    cpl_test(cpl_mask_get_data_const(m)[4] == CPL_BINARY_0);
+    cpl_test(cpl_mask_get_data_const(m)[5] == CPL_BINARY_0);
+    cpl_test(cpl_mask_get_data_const(m)[6] == CPL_BINARY_0);
+    cpl_test(cpl_mask_get_data_const(m)[7] == CPL_BINARY_1);
+    cpl_test(cpl_mask_get_data_const(m)[8] == CPL_BINARY_1);
+    cpl_image_delete(c);
+
+    /* Check that the following holds A/B = C, where
+     *     [ 6 -7  8]       1   [6 7 8]           [ 1 -1  1]
+     * A = [-3  4 -5]  B = -- * [3 4 5]  C = 10 * [-1  1 -1]
+     *     [ 0 -1  2]      10   [9 1 2]           [ 0 -1  1]
+     * The resultant bad pixel masks should be (X indicates bad, '.' good):
+     *         [. . .]
+     * C_bpm = [. . .]
+     *         [. . .]
+     */
+    data_b[6] = 0.6; data_b[7] = 0.7; data_b[8] = 0.8;
+    data_b[3] = 0.3; data_b[4] = 0.4; data_b[5] = 0.5;
+    data_b[0] = 0.9; data_b[1] = 0.1; data_b[2] = 0.2;
+    c = cpl_image_divide_create(a, b);
+    cpl_test_nonnull(c);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[0],   0.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[1], -10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[2],  10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[3], -10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[4],  10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[5], -10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[6],  10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[7], -10.0, FLT_EPSILON);
+    cpl_test_abs(cpl_image_get_data_float_const(c)[8],  10.0, FLT_EPSILON);
+    /* Check that there are no bad pixels. Note, we check this using a slightly
+     * different method from above to test also cpl_image_is_rejected. */
+    m = cpl_image_get_bpm_const(c);
+    cpl_test_null(m);
+    cpl_test_zero(cpl_image_is_rejected(c, 1, 1));
+    cpl_test_zero(cpl_image_is_rejected(c, 2, 1));
+    cpl_test_zero(cpl_image_is_rejected(c, 3, 1));
+    cpl_test_zero(cpl_image_is_rejected(c, 1, 2));
+    cpl_test_zero(cpl_image_is_rejected(c, 2, 2));
+    cpl_test_zero(cpl_image_is_rejected(c, 3, 2));
+    cpl_test_zero(cpl_image_is_rejected(c, 1, 3));
+    cpl_test_zero(cpl_image_is_rejected(c, 2, 3));
+    cpl_test_zero(cpl_image_is_rejected(c, 3, 3));
+    cpl_image_delete(c);
+
+    cpl_image_delete(a);
+    cpl_image_delete(b);
+}
