@@ -69,6 +69,10 @@ static void cpl_image_filter_test(int, int, int, int);
 static void cpl_image_filter_test_5198(void);
 #endif
 
+static void cpl_image_filter_test_5320(void);
+
+static void cpl_image_filter_test_1d(void);
+
 /*-----------------------------------------------------------------------------
                                   Main
  -----------------------------------------------------------------------------*/
@@ -77,6 +81,7 @@ int main(void)
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
     /* Insert tests below */
+    cpl_image_filter_test_5320();
 #ifndef CPL_MEDIAN_SAMPLE
     cpl_image_filter_test_5198();
 #endif
@@ -91,6 +96,8 @@ int main(void)
     cpl_image_filter_test(31, 31, 2, 2);
     cpl_image_filter_test(15, 63, 1, 1);
 
+    cpl_image_filter_test_1d();
+
     return cpl_test_end(0);
 }
 
@@ -205,7 +212,7 @@ static void cpl_image_filter_test(int nx, int ny, int hm, int hn)
                         = cpl_image_new(nx, ny, pixel_type[itype2]);
 
                     cpl_error_code error;
-                    double tol = 0.0;
+                    double tol = itype1 == CPL_TYPE_FLOAT ? 5e-05 : 0.0;
 
 
                     cpl_msg_info(cpl_func, "%u <=> %u - filtering %u X %u %s "
@@ -278,7 +285,7 @@ static void cpl_image_filter_test(int nx, int ny, int hm, int hn)
                                                                       &is_bad);
                                 if (!is_bad) {
                                     cpl_test_rel(stdev, filtered,
-                                                 DBL_EPSILON);
+                                                 4.0 * DBL_EPSILON);
                                 }
                             }
 
@@ -494,3 +501,138 @@ static void cpl_image_filter_test_5198(void)
     cpl_mask_delete(m);
 }
 #endif
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief  Verify the isotropy of linear/average filtering
+  @return void
+  @see Ticket PIPE-5320
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_filter_test_5320(void)
+{
+    cpl_error_code code;
+    /* The dimensions of the matrix and image/mask objects are
+       specified differently! */
+    cpl_matrix * kernel  = cpl_matrix_new(91, 1);
+    cpl_mask   * kernel2 = cpl_mask_new(1, 91);
+
+    cpl_image * unfiltered = cpl_image_load("unfiltered_image.fits",
+                                            CPL_TYPE_UNSPECIFIED, 0, 0);
+    cpl_image * filtered;
+    cpl_image * filtered2;
+
+    double tol = 0.0;
+
+    int itheta;
+    int i, ifilter = CPL_FILTER_LINEAR;
+
+
+    if (unfiltered == NULL) {
+        cpl_msg_info(cpl_func, "Using noise image for test 5320");
+        cpl_test_error(CPL_ERROR_FILE_IO);
+
+        unfiltered = cpl_image_new(400, 400, CPL_TYPE_DOUBLE);
+
+        code = cpl_image_fill_noise_uniform(unfiltered, 0.0, 1.0);
+        cpl_test_eq_error(code, CPL_ERROR_NONE);
+        tol = 8.0 * DBL_EPSILON;
+    } else {
+        cpl_test_error(CPL_ERROR_NONE);
+    }
+
+    filtered   = cpl_image_duplicate(unfiltered);
+    filtered2  = cpl_image_duplicate(unfiltered);
+
+    code = cpl_matrix_fill(kernel, 1.0);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_not(kernel2);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_filter(filtered, unfiltered, kernel,
+                            CPL_FILTER_LINEAR, CPL_BORDER_FILTER);
+
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_filter_mask(filtered2, unfiltered, kernel2,
+                                 CPL_FILTER_AVERAGE, CPL_BORDER_FILTER);
+
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(filtered, filtered2, 6.0 * DBL_EPSILON);
+
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_filter_mask(filtered2, unfiltered, kernel2,
+                                 CPL_FILTER_AVERAGE_FAST, CPL_BORDER_FILTER);
+
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(filtered, filtered2, 256.0 * DBL_EPSILON);
+
+    for (i = 0; i < 2; i +=1, ifilter = CPL_FILTER_AVERAGE) {
+        for (itheta = 0; itheta < 4; itheta +=2) {
+
+            code = cpl_image_flip(unfiltered, itheta);
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+            code = i == 0 ? cpl_image_filter(filtered2, unfiltered, kernel,
+                                             ifilter, CPL_BORDER_FILTER)
+                :  cpl_image_filter_mask(filtered2, unfiltered, kernel2,
+                                         ifilter, CPL_BORDER_FILTER);
+
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+            code = cpl_image_flip(filtered2, itheta);
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+            cpl_test_image_abs(filtered, filtered2, tol);
+
+            code = cpl_image_flip(unfiltered, itheta);
+            cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+        }
+    }
+
+    cpl_matrix_delete(kernel);
+    cpl_mask_delete(kernel2);
+    cpl_image_delete(filtered);
+    cpl_image_delete(filtered2);
+    cpl_image_delete(unfiltered);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief  Verify a case of 1D-filtering, a running median
+  @return void
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_filter_test_1d(void)
+{
+    /* The length of the 1D-image from which to compute the running median */
+    const int nx = IMAGE_SIZE_X;
+    const int hsz = 15; /* Median half-size */
+    const int winnx = 2 * hsz + 1; /* The running window size */
+
+    cpl_image * raw = cpl_image_new(nx, 1, CPL_TYPE_FLOAT);
+    cpl_image * filtered = cpl_image_new(nx - 2 * hsz, 1, CPL_TYPE_FLOAT);
+    cpl_mask * kernel = cpl_mask_new(winnx, 1); 
+
+    cpl_error_code code = cpl_image_fill_noise_uniform(raw, 0.0, 1.0);
+
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_mask_not(kernel);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    code = cpl_image_filter_mask(filtered, raw, kernel, CPL_FILTER_MEDIAN,
+                                 CPL_BORDER_CROP);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
+
+    cpl_mask_delete(kernel);
+    cpl_image_delete(raw);
+    cpl_image_delete(filtered);
+}
