@@ -1886,6 +1886,13 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
                                                 * DBL_EPSILON);
 
                         cpl_polynomial_delete(self0p);
+                    } else if (!error1p && error == CPL_ERROR_SINGULAR_MATRIX) {
+                        cpl_test_error(CPL_ERROR_NONE);
+                        cpl_msg_warning(cpl_func, "Adding a dummy-dimension to "
+                                        "%" CPL_SIZE_FORMAT "-degree, %"
+                                        CPL_SIZE_FORMAT " dimension polynomial "
+                                        "removes matrix singularity",
+                                        degree, mdim);
                     } else {
                         cpl_test_eq_error(error1p, error);
                     }
