@@ -1,4 +1,4 @@
-/* $Id: cpl_fit.h,v 1.10 2011/07/27 14:18:36 llundin Exp $
+/* $Id: cpl_fit.h,v 1.10 2011-07-27 14:18:36 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/07/27 14:18:36 $
+ * $Date: 2011-07-27 14:18:36 $
  * $Revision: 1.10 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifndef CPL_FIT_H
@@ -105,6 +105,7 @@ cpl_error_code cpl_fit_image_gaussian(const cpl_image*, const cpl_image*,
                                       double*,          double*,
                                       double*,          cpl_matrix**);
 
+double cpl_gaussian_eval_2d(const cpl_array *, double, double);
 
 CPL_END_DECLS
 
