@@ -34,14 +34,15 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdlib.h>
-#include <math.h>
-#include <assert.h>
+#include "cpl_photom.h"
 
+#include "cpl_error_impl.h"
 #include "cpl_tools.h"
 #include "cpl_math_const.h"
 
-#include "cpl_photom.h"
+#include <stdlib.h>
+#include <math.h>
+#include <assert.h>
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -158,15 +159,16 @@ cpl_error_code cpl_photom_fill_blackbody(cpl_vector * spectrum,
                 break;
             }
             default:
-                return cpl_error_set(cpl_func, CPL_ERROR_UNSUPPORTED_MODE);
+                return cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
             }
 
             wbeta  *= CPL_PHYS_C;
 
             for (i=0; i < n; i++) {
-                cpl_ensure_code(wlf[i] > 0.0, CPL_ERROR_ILLEGAL_INPUT);
+                if (wlf[i] <= 0.0)
+                    return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
                 sp[i] = walpha / (cpl_tools_ipow(wlf[i], ipow)
-                                  * (exp(wbeta / wlf[i]) - 1.0));
+                                  * (expm1(wbeta / wlf[i])));
             }
 
             break;
@@ -185,19 +187,20 @@ cpl_error_code cpl_photom_fill_blackbody(cpl_vector * spectrum,
                 break;
             }
             default:
-                return cpl_error_set(cpl_func, CPL_ERROR_UNSUPPORTED_MODE);
+                return cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
             }
 
             for (i=0; i < n; i++) {
-                cpl_ensure_code(wlf[i] > 0.0, CPL_ERROR_ILLEGAL_INPUT);
+                if (wlf[i] <= 0.0)
+                    return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
                 sp[i] = walpha * cpl_tools_ipow(wlf[i], ipow)
-                    / (exp(wbeta * wlf[i]) - 1.0);
+                    / (expm1(wbeta * wlf[i]));
             }
 
             break;
         }
         default:
-            return cpl_error_set(cpl_func, CPL_ERROR_UNSUPPORTED_MODE);
+            return cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
     }
 
     return CPL_ERROR_NONE;
