@@ -40,8 +40,6 @@
 #include <string.h>
 #include <fitsio.h>
 
-#include <cxstring.h>
-
 #include <cpl_memory.h>
 #include <cpl_tools.h>
 #include <cpl_error_impl.h>
@@ -958,10 +956,8 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
 
             for (j = 2; j > 0; --j)
             {
-                cx_string *cd = cx_string_new();
-
-                cx_string_sprintf(cd, "CD%1d_%1d", i, j);
-                cpl_property *_p = cpl_property_new(cx_string_get(cd),
+                char *cd = cpl_sprintf("CD%1d_%1d", i, j);
+                cpl_property *_p = cpl_property_new(cd,
                                                     CPL_TYPE_DOUBLE);
 
                 cpl_property_set_double(_p,
@@ -969,7 +965,7 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
                 cpl_property_set_comment(_p, "Coordinate transformation matrix "
                                          "element");
 
-                cx_string_delete(cd);
+                cpl_free(cd);
 
                 cpl_propertylist_insert_after_property(*olist, "CRPIX2", _p);
                 cpl_property_delete(_p);
