@@ -123,7 +123,7 @@ int main(void)
 
 
     /*
-     * Test 3: Copy a parameter
+     * Test 4: Copy a parameter
      */
 
     p = cpl_parameter_new_enum("a", CPL_TYPE_STRING, "A string enum.",
@@ -180,7 +180,7 @@ int main(void)
 
 
     /*
-     * Test 4: Test some error handling
+     * Test 5: Test some error handling
      */
 
     p = cpl_parameter_new_value("a", CPL_TYPE_STRING, "A string value.",
@@ -199,6 +199,27 @@ int main(void)
     cpl_parameter_delete(p);
 
 
+    p = cpl_parameter_new_range("a", CPL_TYPE_BOOL, "A boolean range.",
+                                "None", FALSE, FALSE, TRUE);
+    cpl_test_null(p);
+    cpl_test_error(CPL_ERROR_INVALID_TYPE);
+
+    cpl_parameter_delete(p);
+
+    p = cpl_parameter_new_range("a", CPL_TYPE_STRING, "A string range.",
+                                "None", "aaa", "aaa", "zzz");
+    cpl_test_null(p);
+    cpl_test_error(CPL_ERROR_INVALID_TYPE);
+
+    cpl_parameter_delete(p);
+
+    p = cpl_parameter_new_enum("a", CPL_TYPE_BOOL, "A boolean enumeration.",
+                                "None", FALSE, 2, FALSE, TRUE);
+    cpl_test_null(p);
+    cpl_test_error(CPL_ERROR_INVALID_TYPE);
+
+    cpl_parameter_delete(p);
+
     /*
      * All tests finished
      */
