@@ -1,4 +1,4 @@
-/* $Id: cxmessages.c,v 1.10 2013/02/08 09:11:27 rpalsa Exp $
+/* $Id: cxmessages.c,v 1.10 2013-02-08 09:11:27 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
  * Copyright (C) 2001-2011 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: rpalsa $
- * $Date: 2013/02/08 09:11:27 $
+ * $Date: 2013-02-08 09:11:27 $
  * $Revision: 1.10 $
- * $Name: cpl-6_3_1 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -447,7 +447,7 @@ cx_log_fallback_handler(const cxchar *name, cx_log_level_flags level,
 
 
     /* Keep complier quiet */
-    data = NULL;
+    (void) data;
 
     if (!message)
         message = "(NULL) message";
